@@ -36,36 +36,38 @@
 #endif
 
 #ifdef HAVE_OPENGL
-# ifdef __APPLE__
-#  include <OpenGL/gl.h>
-#  include <OpenGL/glext.h>
-#  include <OpenGL/glu.h>
-//TODO clean this ! chez apple ???
-#  define GL_TEXTURE_RECTANGLE_NV GL_TEXTURE_RECTANGLE_EXT
-# else
-#  include <GL/gl.h>
-#  include <GL/glu.h>
-# endif
+	#if defined(__APPLE__) || defined(OPENGL_HEADER_DIRECTORY_OPENGL)
+		#include <OpenGL/gl.h>
+		#include <OpenGL/glext.h>
+		#include <OpenGL/glu.h>
+		#define GL_TEXTURE_RECTANGLE_NV GL_TEXTURE_RECTANGLE_EXT
+	#else
+		#include <GL/gl.h>
+		#include <GL/glu.h>
+	#endif
 #endif
 
 #ifdef WIN32
-#include <GL/glext.h>
+	#include <GL/glext.h>
 #endif
 
+//extern "C" { SDL_PixelFormat *SDL_AllocFormat(int bpp, Uint32 Rmask, Uint32 Gmask, Uint32 Bmask, Uint32 Amask); }
+
 namespace GAGCore
 {
 	// static local pointer to the actual graphic context
 	static GraphicContext *_gc = NULL;
+	static SDL_PixelFormat _glFormat;
 	
 	// Color
 	Uint32 Color::pack() const
 	{
-		return SDL_MapRGB(_gc->sdlsurface->format, r, g, b) | (a << 24);
+		return (SDL_MapRGB(&_glFormat, r, g, b) & 0x00ffffff) | (a << 24);
 	}
 	
 	void  Color::unpack(const Uint32 packedValue)
 	{
-		SDL_GetRGB(packedValue, _gc->sdlsurface->format, &r, &g, &b);
+		SDL_GetRGB(packedValue, &_glFormat, &r, &g, &b);
 		a = packedValue >> 24;
 	}
 	
@@ -187,6 +189,21 @@ namespace GAGCore
 	} glState;
 	#endif
 	
+	SDL_Surface *DrawableSurface::convertForUpload(SDL_Surface *source)
+	{
+		SDL_Surface *dest;
+		if (_gc->sdlsurface->format->BitsPerPixel == 32)
+		{
+			dest = SDL_DisplayFormatAlpha(source);
+		}
+		else
+		{
+			dest = SDL_ConvertSurface(source, &_glFormat, SDL_SWSURFACE | SDL_SRCALPHA);
+		}
+		assert(dest);
+		return dest;
+	}
+	
 	// Drawable surface
 	DrawableSurface::DrawableSurface(const char *imageFileName)
 	{
@@ -215,7 +232,7 @@ namespace GAGCore
 	{
 		assert(sourceSurface);
 		// beurk, const cast here becasue SDL API sucks
-		sdlsurface = SDL_DisplayFormatAlpha(const_cast<SDL_Surface *>(sourceSurface));
+		sdlsurface = convertForUpload(const_cast<SDL_Surface *>(sourceSurface));
 		assert(sdlsurface);
 		setClipRect();
 		allocateTexture();
@@ -332,11 +349,7 @@ namespace GAGCore
 		if (sdlsurface)
 			SDL_FreeSurface(sdlsurface);
 		
-		Uint32 rmask = _gc->sdlsurface->format->Rmask;
-		Uint32 gmask = _gc->sdlsurface->format->Gmask;
-		Uint32 bmask = _gc->sdlsurface->format->Bmask;
-		Uint32 amask = 0xff000000;
-		sdlsurface = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 32, rmask, gmask, bmask, amask);
+		sdlsurface = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 32, _glFormat.Rmask, _glFormat.Gmask, _glFormat.Bmask, _glFormat.Amask);
 		assert(sdlsurface);
 		setClipRect();
 		initTextureSize();
@@ -394,8 +407,7 @@ namespace GAGCore
 				{
 					if (sdlsurface)
 						SDL_FreeSurface(sdlsurface);
-					sdlsurface = SDL_DisplayFormatAlpha(loadedSurface);
-					assert(sdlsurface);
+					sdlsurface = convertForUpload(loadedSurface);
 					SDL_FreeSurface(loadedSurface);
 					setClipRect();
 					dirty = true;
@@ -1638,7 +1650,7 @@ namespace GAGCore
 		if (flags & USEGPU)
 		{
 			SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
-			sdlFlags |= SDL_OPENGL;
+			sdlFlags |= SDL_OPENGL | SDL_ANYFORMAT;
 		}
 		#else
 		// remove GL from options
@@ -1649,6 +1661,42 @@ namespace GAGCore
 		sdlsurface = SDL_SetVideoMode(w, h, 32, sdlFlags);
 		_gc = this;
 		
+		// set _glFormat
+		if ((optionFlags & USEGPU) && (_gc->sdlsurface->format->BitsPerPixel != 32))
+		{
+			_glFormat.palette = NULL;
+			_glFormat.BitsPerPixel = 32;
+			_glFormat.BytesPerPixel = 4;
+			#if SDL_BYTEORDER == SDL_BIG_ENDIAN
+			_glFormat.Rmask = 0x000000ff;
+			_glFormat.Rshift = 0;
+			_glFormat.Gmask = 0x0000ff00;
+			_glFormat.Gshift = 8;
+			_glFormat.Bmask = 0x00ff0000;
+			_glFormat.Bshift = 16;
+			#else
+			_glFormat.Rmask = 0x00ff0000;
+			_glFormat.Rshift = 16;
+			_glFormat.Gmask = 0x0000ff00;
+			_glFormat.Gshift = 8;
+			_glFormat.Bmask = 0x000000ff;
+			_glFormat.Bshift = 0;
+			#endif
+			_glFormat.Amask = 0xff000000;
+			_glFormat.Ashift = 24;
+			_glFormat.Rloss = 0;
+			_glFormat.Gloss = 0;
+			_glFormat.Bloss = 0;
+			_glFormat.Aloss = 0;
+			_glFormat.colorkey = 0;
+			_glFormat.alpha = 255;
+			//_glFormat = *SDL_AllocFormat(32, 0x00ff0000, 0x0000ff00, 0x000000ff, 0xff000000);
+		}
+		else
+		{
+			memcpy(&_glFormat, _gc->sdlsurface->format, sizeof(SDL_PixelFormat));
+		}
+		
 		// check surface
 		if (!sdlsurface)
 		{
