@@ -64,7 +64,8 @@ Unit::Unit(int x, int y, Uint16 gid, Sint32 typeNum, Team *team, int level)
 	{
 		this->performance[i]=race->getUnitType(typeNum, level)->performance[i];
 		this->level[i]=level;
-		this->canLearn[i]=(bool)race->getUnitType(typeNum, 1)->performance[i]; //TODO: is is a better way to hack this?
+		this->canLearn[i]=(bool)race->getUnitType(typeNum, 3)->performance[i]; //TODO: is is a better way to hack this?
+		// This hack prevent units from unlearning. Units level 3 must have all the abilities of all preceedings levels
 	}
 	
 	experience = 0;
@@ -82,6 +83,7 @@ Unit::Unit(int x, int y, Uint16 gid, Sint32 typeNum, Team *team, int level)
 
 	targetX=0;
 	targetY=0;
+	magicActionTimeout=0;
 
 	// trigger parameters
 	hp=0;
@@ -94,7 +96,7 @@ Unit::Unit(int x, int y, Uint16 gid, Sint32 typeNum, Team *team, int level)
 
 	// warriors wait more tiem before going to eat
 	hungry=HUNGRY_MAX;
-	if (this->performance[ATTACK_SPEED])
+	if (performance[ATTACK_SPEED])
 		trigHungry=(hungry*2)/10;
 	else
 		trigHungry=hungry/4;
@@ -117,6 +119,7 @@ Unit::Unit(int x, int y, Uint16 gid, Sint32 typeNum, Team *team, int level)
 	
 	// gui
 	levelUpAnimation = 0;
+	magicActionAnimation = 0;
 	
 	// debug vars:
 	verbose=false;
@@ -155,6 +158,11 @@ void Unit::load(GAGCore::InputStream *stream, Team *owner, Sint32 versionMinor)
 	action = (Abilities)stream->readUint32("action");
 	targetX = (Sint32)stream->readSint32("targetX");
 	targetY = (Sint32)stream->readSint32("targetY");
+	
+	if (versionMinor >= 41)
+		magicActionTimeout = stream->readSint32("magicActionTimeout");
+	else
+		magicActionTimeout = 0;
 
 	// trigger parameters
 	hp = stream->readSint32("hp");
@@ -171,11 +179,20 @@ void Unit::load(GAGCore::InputStream *stream, Team *owner, Sint32 versionMinor)
 	stream->readEnterSection("abilities");
 	for (int i=0; i<NB_ABILITY; i++)
 	{
-		stream->readEnterSection(i);
-		performance[i] = stream->readSint32("performance");
-		level[i] = stream->readSint32("level");
-		canLearn[i] = (bool)stream->readUint32("canLearn");
-		stream->readLeaveSection();
+		if ((versionMinor < 41) && (i >= 10) && (i < 15))
+		{
+			performance[i] = race->getUnitType(typeNum, 0)->performance[i];
+			level[i] = 0;
+			canLearn[i] = (bool)race->getUnitType(typeNum, 1)->performance[i];
+		}
+		else
+		{
+			stream->readEnterSection(i);
+			performance[i] = stream->readSint32("performance");
+			level[i] = stream->readSint32("level");
+			canLearn[i] = (bool)stream->readUint32("canLearn");
+			stream->readLeaveSection();
+		}
 	}
 	stream->readLeaveSection();
 	
@@ -191,12 +208,16 @@ void Unit::load(GAGCore::InputStream *stream, Team *owner, Sint32 versionMinor)
 	}
 
 	destinationPurprose = stream->readSint32("destinationPurprose");
+	if (versionMinor < 41 && destinationPurprose >= 10)
+		destinationPurprose += 5;
+	
 	subscribed = (bool)stream->readUint32("subscribed");
 
 	caryedRessource = stream->readSint32("caryedRessource");
 	
 	// gui
 	levelUpAnimation = 0;
+	magicActionAnimation = 0;
 	
 	verbose = false;
 	
@@ -234,6 +255,7 @@ void Unit::save(GAGCore::OutputStream *stream)
 	stream->writeUint32((Uint32)action, "action");
 	stream->writeSint32(targetX, "targetX");
 	stream->writeSint32(targetY, "targetY");
+	stream->writeSint32(magicActionTimeout, "magicActionTimeout");
 
 	// trigger parameters
 	stream->writeSint32(hp, "hp");
@@ -512,6 +534,8 @@ void Unit::syncStep(void)
 	// gui
 	if (levelUpAnimation > 0)
 		levelUpAnimation--;
+	if (magicActionAnimation > 0)
+		magicActionAnimation--;
 }
 
 void Unit::selectPreferedMovement(void)
@@ -600,6 +624,96 @@ void Unit::stopAttachedForBuilding(bool goingInside)
 	assert(needToRecheckMedical);
 }
 
+void Unit::handleMagic(void)
+{
+	assert(medical==MED_FREE);
+	assert((displacement!=DIS_ENTERING_BUILDING) && (displacement!=DIS_INSIDE) && (displacement!=DIS_EXITING_BUILDING));
+	
+	magicActionTimeout--;
+	if (magicActionTimeout > 0)
+		return;
+	
+	Map *map = &owner->game->map;
+	Team **teams = owner->game->teams;
+	
+	bool hasUsedMagicAction = false;
+	if (performance[MAGIC_ATTACK_AIR] || performance[MAGIC_ATTACK_GROUND])
+	{
+		for (int yi=posY-3; yi<=posY+3; yi++)
+			for (int xi=posX-3; xi<=posX+3; xi++)
+			{
+				// damaging enemy units:
+				for (int altitude=0; altitude<2; altitude++)
+				{
+					Uint16 targetGUID;
+					Sint32 attackForce;
+					if ((altitude == 1) && performance[MAGIC_ATTACK_AIR])
+					{
+						targetGUID = map->getAirUnit(xi, yi);
+						attackForce = performance[MAGIC_ATTACK_AIR];
+					}
+					else if ((altitude == 0) && performance[MAGIC_ATTACK_GROUND])
+					{
+						targetGUID = map->getGroundUnit(xi, yi);
+						attackForce = performance[MAGIC_ATTACK_GROUND];
+					}
+					else
+						continue;
+					if (targetGUID != NOGUID)
+					{
+						Sint32 targetTeam = Unit::GIDtoTeam(targetGUID);
+						Uint16 targetID = Unit::GIDtoID(targetGUID);
+						Uint32 targetTeamMask = 1<<targetTeam;
+						if (owner->enemies & targetTeamMask)
+						{
+							Unit *enemyUnit = teams[targetTeam]->myUnits[targetID];
+							Sint32 damage = attackForce + experienceLevel - enemyUnit->performance[ARMOR];
+							if (damage > 0)
+							{
+								enemyUnit->hp -= damage;
+								enemyUnit->owner->setEvent(xi, yi, Team::UNIT_UNDER_ATTACK_EVENT, targetGUID);
+								incrementExperience(damage);
+								magicActionAnimation = MAGIC_ACTION_ANIMATION_FRAME_COUNT;
+								hasUsedMagicAction = true;
+							}
+						}
+					}
+				}
+				
+				// damaging enemy buildings:
+				if (performance[MAGIC_ATTACK_GROUND])
+				{
+					Uint16 targetGBID = map->getBuilding(xi, yi);
+					if (targetGBID != NOGBID)
+					{
+						Sint32 targetTeam = Building::GIDtoTeam(targetGBID);
+						Uint16 targetID = Building::GIDtoID(targetGBID);
+						Uint32 targetTeamMask = 1<<targetTeam;
+						if (owner->enemies & targetTeamMask)
+						{
+							Building *enemyBuilding = teams[targetTeam]->myBuildings[targetID];
+							Sint32 damage = performance[MAGIC_ATTACK_GROUND] + experienceLevel - enemyBuilding->type->armor;
+							if (damage > 0)
+							{
+								enemyBuilding->hp -= damage;
+								enemyBuilding->owner->setEvent(xi, yi, Team::BUILDING_UNDER_ATTACK_EVENT, targetGBID);
+								if (enemyBuilding->hp <= 0)
+									enemyBuilding->kill();
+								incrementExperience(damage);
+								magicActionAnimation = MAGIC_ACTION_ANIMATION_FRAME_COUNT;
+								hasUsedMagicAction = true;
+							}
+						}
+					}
+				}
+			}
+		
+		Sint32 magicLevel = std::max(level[MAGIC_ATTACK_AIR], level[MAGIC_ATTACK_GROUND]);
+		if (hasUsedMagicAction)
+			magicActionTimeout = race->getUnitType(typeNum, level[magicLevel])->magicActionCooldown;
+	}
+}
+
 void Unit::handleMedical(void)
 {
 	if ((displacement==DIS_ENTERING_BUILDING) || (displacement==DIS_INSIDE) || (displacement==DIS_EXITING_BUILDING))
@@ -663,6 +777,8 @@ void Unit::handleActivity(void)
 	
 	if (medical==MED_FREE)
 	{
+		handleMagic();
+		
 		if (activity==ACT_RANDOM)
 		{
 			// look for a "job"
@@ -773,8 +889,8 @@ void Unit::handleActivity(void)
 				printf("guid=(%d) no job found.\n", gid);
 			
 			// nothing to do:
-			// we go to a heal building if we'r not fully healed:
-			if (hp<performance[HP])
+			// we go to a heal building if we'r not fully healed: (1/8 trigger)
+			if (hp+(performance[HP]/10) < performance[HP])
 			{
 				Building *b;
 				b=owner->findNearestHeal(this);
@@ -1377,16 +1493,19 @@ void Unit::handleDisplacement(void)
 				if (destinationPurprose==FEED)
 				{
 					insideTimeout=-attachedBuilding->type->timeToFeedUnit;
+					speed=attachedBuilding->type->insideSpeed;
 				}
 				else if (destinationPurprose==HEAL)
 				{
+					//insideTimeout=-(attachedBuilding->type->timeToHealUnit*(performance[HP]-hp))/performance[HP];
 					insideTimeout=-attachedBuilding->type->timeToHealUnit;
+					speed=(attachedBuilding->type->insideSpeed*performance[HP])/(performance[HP]-hp);
 				}
 				else
 				{
 					insideTimeout=-attachedBuilding->type->upgradeTime[destinationPurprose];
+					speed=attachedBuilding->type->insideSpeed;
 				}
-				speed=attachedBuilding->type->insideSpeed;
 			}
 			else if (displacement==DIS_INSIDE)
 			{
